@@ -3,8 +3,8 @@
 
 Copyright (c) 2003 Terence Way <terry@wayforward.net>
 Portions Copyright(c) 2004,2005,2006,2007,2008,2011,2012 Stuart Gathman <stuart@bmsi.com>
-Portions Copyright(c) 2005,2006,2007,2008,2011,2012,2013 Scott Kitterman <scott@kitterman.com>
-Portions Copyright(c) 2013 Stuart Gathman <stuart@gathman.org>
+Portions Copyright(c) 2005,2006,2007,2008,2011,2012,2013,2014 Scott Kitterman <scott@kitterman.com>
+Portions Copyright(c) 2013,2014 Stuart Gathman <stuart@gathman.org>
 
 This module is free software, and you may redistribute it and/or modify
 it under the same terms as Python itself, so long as this copyright message
@@ -30,33 +30,41 @@ For news, bugfixes, etc. visit the home page for this implementation at
     http://www.wayforward.net/spf/
 """
 
-# CVS Commits since last release (2.0.9):
+# CVS Commits since last release (2.0.10):
 # $Log: spf.py,v $
-# Revision 1.108.2.128  2014/09/02 17:31:53  customdesigned
-# Release 2.0.10
+# Revision 1.108.2.136  2014/12/05 16:20:07  customdesigned
+# Release 2.0.11
 #
-# Revision 1.108.2.127  2014/09/01 21:17:13  kitterma
-# Fix TempError handling of errors from the DNS module.
+# Revision 1.108.2.135  2014/12/03 01:11:09  customdesigned
+# Fold case of domain for all cache entries.
 #
-# Revision 1.108.2.126  2014/08/02 18:35:50  customdesigned
-# '~' is also an unreserved char in rfc7208.
+# Revision 1.108.2.134  2014/12/03 01:01:24  customdesigned
+# PTR case change fix with test case
 #
-# Revision 1.108.2.125  2014/08/02 04:36:48  kitterma
-#   * Fix bug in SPF record parsing that caused all 'whitespace' characters to
-#     be considered valid term separators and not just spaces
+# Revision 1.108.2.133  2014/10/06 11:54:11  kitterma
+# *** empty log message ***
 #
-# Revision 1.108.2.124  2014/08/02 04:32:36  kitterma
-# Archive previous commit messages for spf.py in pyspf_changelog.txt and bumpi
-# version to 2.0.10 for start of follow on work.
+# Revision 1.108.2.132  2014/10/06 11:51:03  kitterma
+#   * Downcase IPv6 PTR results since case inconsistency can cause PTR match
+#     failures (patch thanks to Joni Fieggen)
+#
+# Revision 1.108.2.131  2014/09/22 17:20:33  customdesigned
+# Update comments
+#
+# Revision 1.108.2.130  2014/09/22 17:13:53  customdesigned
+# Cleaner fix for multiple spaces.
+#
+# Revision 1.108.2.129  2014/09/21 21:11:47  kitterma
+#  * Reset to start 2.0.11 development
+#  * Fixed bug where multiple spaces between terms causes pyspf to think they
+#    were unknown mechanisms
 #
-# Revision 1.108.2.123  2014/07/30 18:41:18  customdesigned
-# Fix flagging AAAA records in dns_a.  Add --strict option to CLI
 #
 # See pyspf_changelog.txt for earlier CVS commits.
 
 __author__ = "Terence Way, Stuart Gathman, Scott Kitterman"
 __email__ = "pyspf@openspf.org"
-__version__ = "2.0.10: Sep 2, 2014"
+__version__ = "2.0.11: Dec 5, 2014"
 MODULE = 'spf'
 
 USAGE = """To check an incoming mail request:
@@ -776,7 +784,7 @@ class query(object):
 
         # Split string by space, drop the 'v=spf1'.  Split by all whitespace
         # casuses things like carriage returns being treated as valid space
-        # separators, so split() is not sufficient.
+        # separators, so split() is not sufficient.  
         spf = spf.split(' ')
         # Catch case where SPF record has no spaces.
         # Can never happen with conforming dns_spf(), however
@@ -787,7 +795,10 @@ class query(object):
             if self.strict > 1:
                 raise AmbiguityWarning('Invalid SPF record in', self.d)
             return ('none', 250, EXPLANATIONS['none'])
-        spf = spf[1:]
+        # Just to make it even more fun, the relevant piece of the ABNF for
+        # term separations is *( 1*SP ( directive / modifier ) ), so it's one
+        # or more spaces, not just one.  So strip empty mechanisms.
+        spf = [mech for mech in spf[1:] if mech]
 
         # copy of explanations to be modified by exp=
         exps = self.exps
@@ -1257,6 +1268,7 @@ class query(object):
         if name.endswith('.'): name = name[:-1]
         if not reduce(lambda x,y:x and 0 < len(y) < 64, name.split('.'),True):
             return []   # invalid DNS name (too long or empty)
+        name = name.lower()
         result = self.cache.get( (name, qtype), [])
         if result: return result
         cnamek = (name,'CNAME')
@@ -1274,6 +1286,9 @@ class query(object):
                 timeout = self.timeout
             timethen = time.time()
             for k, v in DNSLookup(name, qtype, self.strict, timeout):
+                # Force case insensitivity in cache, DNS servers often
+                # return random case in domain part of answers.
+                k = (k[0].lower(), k[1]) 
                 if k == cnamek:
                     cname = v
                 if k[1] == 'CNAME' or (qtype,k[1]) in safe2cache:
